package Site::SQLContainer;
# LICENSE
#
# Copyright (c) 2001 Steve Slaven. All rights reserved.
#
# This program is free software  and is provided "as is" without
# express or implied warranty. You can redistribute it and/or
# modify it under the same terms as Perl itself.
#
use strict;
use vars qw( @ISA %AUTOFIELDS $AUTOSETTINGS );

use PObject;
@ISA = qw( PObject );

use Site::Page;
use Site::SQLField;

$AUTOSETTINGS = 1;
%AUTOFIELDS = (
			TABLE => undef,
			PKEYS => undef,
			PKEY => undef, # Just to catch the pkey method

			NAMEDFIELDS => undef,
			LISTFIELDS => undef,
			BOUNDFIELDS => undef,

			OPTS => undef,
			DBH => undef,

			CARRY => undef
	);

sub OBJ_NEW
{
	my $Self = shift;

	$Self -> { NAMEDFIELDS } = {};
	$Self -> { BOUNDFIELDS } = {};
	$Self -> { LISTFIELDS } = [];

	$Self -> { CARRY } = {};
	$Self -> { PKEYS } = [];

	if( ! defined( $Self -> { OPTS } ) )
	{
		# Kludge?
		$Self -> { OPTS } = $Site::Page::OPTS;
	}

	return( $Self );
}

sub addfield
{
	my $Self = shift;
	my %FixEm = @_;
	my $Field = Site::SQLField -> new( container => $Self,
				type => $FixEm{ type } );

	for ( keys( %FixEm ) )
	{
		#print $_ . "=" . $FixEm{ $_ } . "\n";
		$Field -> $_( $FixEm{ $_ } );
	}

	# Push the ref also into the listfields
	push( @{ $Self -> { LISTFIELDS } }, $Field );
	return( $Field );
}

sub addfields
{
	my $Self = shift;

	# Just add the fields
	for ( @_ )
	{
		$Self -> addfield( %{ $_ } );
	}
}

sub _HTMLNAME
{
	my $Self = shift;
	return( join( ".", $Self -> Table(), @{ $Self -> PKeys() } ) );
}

sub _ACTION
{
	# The name of my action
	my $Self = shift;
	return( $Self -> _HTMLNAME() . ".__.action" );
}

sub _SELECTFIELDS
{
	my $Self = shift;
	# Return the field names of the fields to mod

	return( grep( /^.+$/, map( $_ -> SQLSelect(),
				@{ $Self -> { LISTFIELDS } } ) ) );

}

sub _MODIFYFIELDS
{
	my $Self = shift;
	# Return the field names of the fields to mod

	return( grep( /^.+$/, map( $_ -> SQLUpdate(),
				@{ $Self -> { LISTFIELDS } } ) ) );

}

sub addcarry
{
	# Add vars to the carryo
	my $Self = shift;
	my %Carry = @_;

	for( keys( %Carry ) )
	{
		$Self -> Carry() -> { $_ } = $Carry{ $_ };
	}
}

# Quickie to set only 1 pkey
sub OBJ_HOOK_PKEY
{
	my $Self = shift;
	$Self -> PKeys( [ shift() ] );
	return( $Self -> PKeys() -> [ -1 ] );
}

sub getboundfield
{
	my $Self = shift;
	return( $Self -> BoundFields() -> { shift() } );
}

sub getnamedfield
{
	my $Self = shift;
	return( $Self -> NamedFields() -> { shift() } );
}

sub getfield
{
	return( shift() -> getnamedfield( shift() ) );
}

sub bind_sql_object
{
	# Takes an SQLObject subclass name and
	# returns a whatever based on it.  Ye!
	my $ClassType = shift;
	my $ObjType = shift;

	my( $Ret, %sp, %field, @PKeys, $Field, %FAT, $IsLocked );

	eval( "use $ObjType" );
	%sp = $ObjType -> SQL_PROPERTIES();

	$Ret = $ClassType -> new();

	# Some copys
	$Ret -> Table( $sp{ table } );
	$Ret -> DBH( $sp{ dbh } );

	# PKeys
	@PKeys = map( $_ -> { name }, @{ $sp{ pkeys } } );
	$Ret -> PKeys( \@PKeys );

	# Now the fields
	for( @{ $sp{ pkeys } }, @{ $sp{ fields } } )
	{
		if( $_ -> { insert } eq "no" )
		{
			$IsLocked = 1;
		}else
		{
			$IsLocked = 0;
		}

		if( $_ -> { fat } )
		{
			%FAT = %{ $_ -> { fat } };
		}else
		{
			%FAT = ();
		}

		$Field = $Ret -> addfield(
				name => $_ -> { name },
				boundto => $_ -> { name },
				locked => $IsLocked,
				enabled => ( ! $IsLocked ),
				%FAT );
	}

	return( $Ret );
}

sub defaultdbh
{
        shift();

        $AUTOFIELDS{ DBH } = shift();
}

sub defaultopts
{
        shift();

        $AUTOFIELDS{ OPTS } = shift();
}

sub _DELETEFIELD
{
	my $Self = shift;
	my $Field = shift;
	my( $Name, $Bound, @NewFields );

	# This is for fields to request to be removed from the
	# @FIELDS array
	$Bound = $Field -> BoundTo();
	$Name = $Field -> Name();

	@NewFields = grep( $_ != $Field, @{ $Self -> ListFields() } );
	$Self -> { LISTFIELDS } = \@NewFields;

	if( $Bound )
	{
		delete( $Self -> BoundFields() -> { $Bound } );
	}

	if( $Name )
	{
		delete( $Self -> NamedFields() -> { $Name } );
	}
}

sub removeexceptnamed
{
	# Removes all but the listed fields
	my $Self = shift;
	my @ToKeep = @_;
	my( $name );

	for( @{ $Self -> ListFields() } )
	{
		$name = $_ -> Name();
		if( ! grep( $_ eq $name, @ToKeep ) )
		{
			$_ -> Remove();
		}
	}
}

# Legacy support
sub BindSQLObject { return( shift() -> bind_sql_object( @_ ) ) }

1;
