package Site::SQLForm;
# LICENSE
#
# Copyright (c) 2001 Steve Slaven. All rights reserved.
#
# This program is free software  and is provided "as is" without
# express or implied warranty. You can redistribute it and/or
# modify it under the same terms as Perl itself.
#

use strict;
use vars qw( @ISA %AUTOFIELDS $AUTOSETTINGS );

use Site::SQLContainer;
@ISA = qw( Site::SQLContainer );

use Site::Page;
use Site::Template;

$AUTOSETTINGS = 1;
%AUTOFIELDS = (
			FORMACTION => $::ENV{ SCRIPT_NAME },
			TEMPLATE => undef,
			DELETETEMPLATE => undef,
			DELETEBUTTON => 'Delete',
			ADDBUTTON => 'Add',
			EDITBUTTON => 'Edit',
			DELETEYESBUTTON => 'Yes',
			DELETENOBUTTON => 'No',
			MESSAGE => undef,
			PKEYVALS => undef,
			USEDATA => undef,
			LASTACTION => undef
	);

sub handle
{
	# Return true if handled something, false if not
	# Will also set BADFORM if I'm bad.  :)
	my $Self = shift;

	# Check for PKEY values and fill them in if applicable
	$Self -> _FillPKeys();
	#$Self -> _MapBoundFields();

	my $Do = $Self -> _WhatToDo();
	my $MagicQ;
	my @FieldNames = $Self -> _MODIFYFIELDS();
	my @Values; # = $Self -> _Values();

	#print "$Do<BR>";

	# Gather fields values
	#for( keys( %{ $Self -> Opts() } ) )
	#{
	#	print $_ . ": " . $Self -> Opts() -> { $_ } . "<BR>";
	#}

	# Do?
	if( ! grep( $Do eq $_, 'UPDATE',
				'ADD',
				'DELETE' ) )
	{
		return( undef );
	}

	for( @FieldNames )
	{
		#print $Self -> getnamedfield( $_ ) -> _HTMLNAME();
		# Set the value in the field
		$Self -> getboundfield( $_ ) -> SetValue( $Self -> Opts() -> 
			{ $Self -> getboundfield( $_ ) -> _HTMLNAME() } );

		push( @Values, $Self -> getboundfield( $_ ) -> SQLValue );
	}

	if( $Do eq "UPDATE" )
	{
		# Perform an update query
		$MagicQ = 'update ' .
				$Self -> Table() .
				' set ' .
				join( ", ", map( $_ . "=?",
					$Self -> _MODIFYFIELDS() ) ) .
				' where ' .
				join( " AND ", map( $_ . "=?",
					@{ $Self -> PKeys() } ) );

		# Add PKEYS to the values array
		for( @{ $Self -> PKeys } )
		{
			push( @Values, $Self -> Opts() -> {
					$Self -> _HTMLNAME() . '.pkeys.' . $_ }
				);
		}

		$Self -> LastAction( "UPDATE" );
	}elsif( $Do eq "ADD" )
	{
		# Perform an insert query
		$MagicQ = 'insert into ' .
			$Self -> Table() .
			' ( ' .
			join( ", ", $Self -> _MODIFYFIELDS() ) .
			' ) values ( ' .
			join( ", ", map( '?', $Self -> _MODIFYFIELDS() ) ) .
			' ) ';

		$Self -> LastAction( "ADD" );
	}elsif( $Do eq "DELETE" )
	{
		# Perform deleteooo
		$MagicQ = 'delete from ' .
				$Self -> Table() . 
				' where ' .
				join( " AND ", map( $_ . "=?",
						@{ $Self -> PKeys() } ) );
		@Values = ();
		for( @{ $Self -> PKeys } )
		{
			push( @Values, $Self -> Opts() -> {
					$Self -> _HTMLNAME() . '.pkeys.' . $_ }
				);
		}
		$Self -> LastAction( "DELETE" );
	}

	#print $MagicQ;
	#print "(" . join( ", ", @Values ) . ")";

	# Do it!
	my $Status = $Self -> DBH() -> do( $MagicQ, undef, @Values );

	if( ! defined( $Status ) )
	{
		# Baddy!
		$Self -> { BADFORM } = 1;
		$Self -> AddMessage( "An SQL error occurred: '" .
				$Self -> DBH() -> errstr() . "'" );
	}

	return( $Do );
}

sub display
{
	my $Self = shift;

	# Decide what to do based on vars
	$Self -> _FillFields();
	#print $Self -> _WhatForm();

	# Once fields are filled, perform field to href bindings
	$Self -> { USEDATA } = {
			begin => $Self -> HTMLBegin(),
			end => $Self -> HTMLEnd(),

			message => $Self -> Message(),

			add => $Self -> HTMLAdd(),
			edit => $Self -> HTMLEdit(),
			delete => $Self -> HTMLDelete(),

			yes => $Self -> HTMLDeleteYes(),
			no => $Self -> HTMLDeleteNo(),

			fields => []
		};
	for( @{ $Self -> ListFields() } )
	{
		$Self -> _AddFieldToHREF( $Self -> UseData(), $_ );
	}

	if( $Self -> _WhatForm() eq "DELETE" )
	{
		$Self -> _DisplayDelete();
	}else
	{
		$Self -> _DisplayForm();
	}
}

sub _WhatToDo
{
	# Look at the opts and decide what to do
	my $Self = shift;
	my $ActVal = $Self -> Opts() -> { $Self -> _ACTION() };
	my $ButVal = $Self -> Opts() -> { $Self -> _Button() };

	#print $ActVal . "::" . $ButVal . "<BR>";
	if( $ButVal eq $Self -> DeleteButton() )
	{
		# If they pushed delete then ask if they are sure
		return( "ASKDELETE" );
	}elsif( $ActVal eq "DELETE" )
	{
		# They said yes?
		if( $ButVal eq $Self -> DeleteYesButton() )
		{
			return( "DELETE" );
		}else
		{
			# Nope
			return( "NODELETE" )
		}
	}elsif( $ActVal eq "UPDATE" )
	{
		return( "UPDATE" );
	}elsif( $ActVal eq "ADD" )
	{
		return( "ADD" );
	}elsif( $Self -> PKeyVals() )
	{
		return( "ASKUPDATE" );
	}else
	{
		# Nothing means askadd
		return( "ASKADD" );
	}
}

sub _WhatForm
{
	# Just a logic filter for the above
	my $Self = shift;
	my $Do = $Self -> _WhatToDo();

	# This will tell what form to display and the method it should do
	if( grep( $_ eq $Do, 'ASKADD',
				'ADD',
				'UPDATE',
				'DELETE',
				'NODELETE' ) )
	{
		return( "ADD" );
	}elsif( $Do eq 'ASKUPDATE' )
	{
		return( "UPDATE" );
	}elsif( $Do eq "ASKDELETE" )
	{
		# Munge the PKEYS out of the query
		$Self -> PKeyVals( [] );
		for( @{ $Self -> PKeys } )
		{
			push( @{ $Self -> PKeyVals() }, $Self -> Opts() -> {
					$Self -> _HTMLNAME() . '.pkeys.' . $_ }
				);
		}
		return( "DELETE" );
	}

	return( undef );
				
}

sub _FillFields
{
	# Call proper filler based on action to take
	my $Self = shift;
	my $Do = $Self -> _WhatForm();

	# If askadd or just updated or just deleted, then fill defaults
	if( $Do eq 'ADD' )
	{
		$Self -> _FillFieldsAdd();
	}elsif( $Do eq "UPDATE" )
	{
		$Self -> _FillFieldsUpdate();
	}elsif( $Do eq "DELETE" )
	{
		$Self -> _FillFieldsUpdate();
	}
}

sub _FillFieldsUpdate
{
	# This will get values from an SQL query
	my $Self = shift();

	my @PKeys = @{ $Self -> PKeys() };
	my @Fields = $Self -> _SELECTFIELDS();
	my @Values;

	# Generate the select query
	my $MagicQ = 'select ' . join( ", ", @Fields ) .
		' from ' . $Self -> Table() .
		' where ' .
		join( " AND ", map( $_ . "=?", @PKeys ) );

	my $sth = $Self -> DBH() -> prepare( $MagicQ );
	$sth -> execute( @{ $Self -> PKeyVals() } );

	my $Ref = $sth -> fetchrow_hashref();

	if( ! defined( $Ref ) )
	{
		$Self -> AddMessage( "Row not found" );
		# Should break it down a bunch here
		return( undef );
	}
	for( keys( %{ $Ref } ) )
	{
		$Self -> getboundfield( $_ ) -> SetValue( $Ref -> { $_ } );
	}
}

sub _FillFieldsAdd
{
	my $Self = shift;

	for( @{ $Self -> HTMLFields() } )
	{
		$_ -> SetDefault();
	}
}

sub HTMLFields
{
	my $Self = shift;
	return( $Self -> ListFields() );
}

sub _FillFieldsBad
{
	# This will get values from the environment
}

sub _DisplayForm
{
	my $Self = shift;

	my $Ret;

	# Performs the actual form display
	if( ! $Self -> Template() )
	{
		$Self -> Template( Site::Template -> new() );
		$Self -> Template() -> Template( $Self -> _DefaultTemplate() );
	}

	$Self -> Template() -> registerdefault( $Self -> UseData() );
	$Self -> Template() -> display();
}

sub _DisplayDelete
{
	my $Self = shift;
	if( ! $Self -> DeleteTemplate() )
	{
		$Self -> DeleteTemplate( Site::Template -> new() );
		$Self -> DeleteTemplate() ->
			Template( $Self -> _DefaultDeleteTemplate() );
	}

	$Self -> DeleteTemplate() -> registerdefault( $Self -> UseData() );
	$Self -> DeleteTemplate() -> display();
}

sub _DefaultTemplate
{
	# Return a default template
	return( qq{<t:var name=begin/>
<PRE>
<t:var name=message/>
</PRE>
<TABLE BORDER=0>
<t:loop name=fields>
<TR>
<TD><t:iftrue name=label><B><t:var name=label/>:</B></t:iftrue></TD>
<TD><t:var name=field/></TD>
</TR>
</t:loop>
</TABLE>
<t:var name=add/>
<t:var name=edit/>
<t:var name=delete/>
<t:var name=end/>
} );

}

sub _DefaultDeleteTemplate
{
	return( qq{<t:var name=begin/>
Delete it?

<TABLE BORDER=0>
<t:loop name=fields>
<TR>
<TD><t:iftrue name=label><B><t:var name=label/>:</B></t:iftrue></TD>
<TD><t:var name=display/></TD>
</TR>
</t:loop>
</TABLE>

<t:var name=yes/>
<t:var name=no/>
<t:var name=end/>
} );
}

sub HTMLBegin
{
	my $Self = shift;
	my $Action = $Self -> FormAction();
	my $Do = $Self -> _WhatForm();

	my $Ret = qq{<FORM ACTION=$Action METHOD=GET>
} . '<INPUT TYPE=HIDDEN NAME=' . $Self -> _ACTION() .
	' VALUE="' . $Do . '">';

	if( $Do ne "ADD" )
	{
		# Add pkey vals
		my @Vals = @{ $Self -> PKeyVals() };
		for( @{ $Self -> PKeys() } )
		{
			$Ret .= '<INPUT TYPE=HIDDEN NAME=' .
				$Self -> _HTMLNAME() .
				'.pkeys.' .
				$_ .
				' VALUE="' .
				Site::Page ->
					encode_entities( shift( @Vals ) ) .
				'">';
		}
	}

	# Spew forth the carrables!
	for ( keys( %{ $Self -> Carry() } ) )
	{
		$Ret .= '<INPUT TYPE=HIDDEN NAME=' .
			$_ .
			' VALUE="' .
			Site::Page ->
				encode_entities( $Self -> Carry() -> { $_ } ) .
			'">';
	}

	return( $Ret );
}


sub HTMLEnd
{
	return( qq{</FORM>} );
}

sub HTMLLabel
{
	my $Self = shift;
	my $Fname = shift;

	return( $Self -> getnamedfield( $Fname ) -> HTMLLabel() );
}

sub HTMLField
{
	my $Self = shift;
	my $Fname = shift;

	return( $Self -> getnamedfield( $Fname ) -> HTMLField() );
}

sub HTMLAdd
{
	# The add button, if applicable
	my $Self = shift;

	if($Self -> _WhatForm() eq "ADD" )
	{
		return( '<INPUT TYPE=SUBMIT NAME=' . 
			$Self -> _Button() .
			' VALUE="' . 
			$Self -> AddButton() .
			'">' );
	}
	return( undef );
}

sub HTMLEdit
{
	# The edit button
	my $Self = shift;
	if( $Self -> _WhatForm() eq "UPDATE" )
	{
		return( '<INPUT TYPE=SUBMIT NAME=' . 
			$Self -> _Button() .
			' VALUE="' . 
			$Self -> EditButton() .
			'">' );
	}

	return( undef );
}

sub HTMLDelete
{
	# The delete button
	# Only on the update form
	my $Self = shift;

	if( $Self -> _WhatForm() eq "UPDATE" )
	{
		return( '<INPUT TYPE=SUBMIT NAME=' . 
			$Self -> _Button . 
			' VALUE="' . $Self -> DeleteButton() . '">' ); 
	}
	return( undef );
}

sub HTMLDeleteYes
{
	my $Self = shift();

	return( '<INPUT TYPE=SUBMIT NAME=' . 
			$Self -> _Button . 
			' VALUE="' . $Self -> DeleteYesButton() . '">' ); 
}

sub HTMLDeleteNo
{
	my $Self = shift();

	return( '<INPUT TYPE=SUBMIT NAME=' . 
			$Self -> _Button . 
			' VALUE="' . $Self -> DeleteNoButton() . '">' ); 
}

sub HTMLDisplay
{
	my $Self = shift;
	return( $Self -> getnamedfield( shift() ) -> displayValue() );
}

sub _Button
{
	my $Self = shift;

	return( $Self -> _HTMLNAME() . ".__.button" );
}

sub AddMessage
{
	my $Self = shift;
	$Self -> { MESSAGE } .= shift() . "\n";
}

sub _FillPKeys
{
	my $Self = shift;
	my @Arr;
	my $IsSet = 0;

	for( @{ $Self -> _HTMLPKeys() } )
	{
		push( @Arr, $Self -> Opts() -> { $_ } );
		if( defined( $Arr[ -1 ] ) )
		{
			$IsSet = 1;
		}
	}

	if( $IsSet )
	{
		$Self -> { PKEYVALS } = \@Arr;
	}
}

sub _HTMLPKeys
{
	# This will be configurable later, for now the pkeys are not fully
	# qualified
	my $Self = shift;

	return( $Self -> PKeys() );
}

sub _AddFieldToHREF
{
        my $Self = shift;
        my $href = shift;
        my $Field = shift;

        my( $useref, $fieldref );
  
        $useref = {
                        field => $Field -> HTMLField(),
                        label => $Field -> HTMLLabel(),
                        display => $Field -> HTMLDisplay(),
                };

        push( @{ $href -> { fields } }, $useref );
        $href -> { names } = $useref;
        $href -> { boundto } = $useref;
}

1;
