package Site::SQLTable;
# LICENSE
#
# Copyright (c) 2001 Steve Slaven. All rights reserved.
#
# This program is free software  and is provided "as is" without
# express or implied warranty. You can redistribute it and/or
# modify it under the same terms as Perl itself.
#
use strict;
use vars qw( @ISA %AUTOFIELDS $AUTOSETTINGS );

use Site::SQLContainer;
@ISA = qw( Site::SQLContainer );

use Site::Page;
use Site::Template;

$AUTOSETTINGS = 1;
%AUTOFIELDS = (
			FORMACTION => $::ENV{ SCRIPT_NAME },
			TEMPLATE => undef,
			DELETETEMPLATE => undef,
			DELETEBUTTON => 'Delete',
			ADDBUTTON => 'Add',
			EDITBUTTON => 'Edit',
			DELETEYESBUTTON => 'Yes',
			DELETENOBUTTON => 'No',
			MESSAGE => undef,
			PKEYVALS => undef,
			WHERE => undef,
			WHEREARGS => undef,
			STARTFROM => 1,
			PERPAGE => 10,
			QUERY => undef,
			ORDER => undef,
			USEDATA => undef
	);

sub OBJ_NEW
{
	my $Self = shift;

	$Self -> { USEDATA } = {
					rows => []
			};

	return( $Self );
}

sub display
{
	my $Self = shift;
	#$Self -> _MapBoundFields();
	$Self -> _LoadTableFromOpts();

	my @ToDisplay; # <--- The array of rows to display for this run... ?

	# Loop through the query.  :)
	my @PKeys = @{ $Self -> PKeys() };
	my @Fields = $Self -> _SELECTFIELDS();
	my @Values;

	# Generate the select query
	my $MagicQ;

	if( $Self -> Query() )
	{
		$MagicQ = $Self -> Query();
	}
	else
	{
		$MagicQ = 'select ' . join( ", ", @Fields ) .
			' from ' . $Self -> Table();

		# Filled in because it's used later
		$Self -> Query( $MagicQ );
	}

	if( $Self -> Where() )
	{
		$MagicQ .=
			' where ' .
			$Self -> Where();
	}

	if( $Self -> Order() )
	{
		$MagicQ .=  ' order by ' . join( ", ",
			@{ $Self -> Order() } );
	}

	my $sth = $Self -> DBH() -> prepare( $MagicQ );
	if( $Self -> WhereArgs() )
	{
		$sth -> execute( @{ $Self -> WhereArgs() } );
	}else
	{
		$sth -> execute();
	}

	my( $ThisRow, $ThisRef, $TheField );
	my $Rows = $sth -> fetchall_arrayref({});
	my( $ThisValue, $FieldsRef, $Tmp );

	# Gotta put in the headers
	$Self -> UseData() -> { headers } = [];
	for( @{ $Self -> ListFields() } )
	{
		push( @{ $Self -> UseData() -> { headers } }, {
				header => $Self -> HTMLHeader( $_ -> Name() )
			} );
	}

	for( @{ $Rows } )
	{
		# Fill fields, then pull vals into ThisRow and push
		# it onto the stack of rows	
		$ThisRef = $_;
		$ThisRow = {};

		# We have to loop through the fields because
		# of the formatting, etc
		for( keys( %{ $ThisRef } ) )
		{
			$TheField = $Self -> getboundfield( $_ );
			$TheField -> SetValue( $ThisRef -> { $_ } );
		}

		# Also, for each unbound field, perform {} substitution on it
		for( @{ $Self -> ListFields() } )
		{
			# Copy/sub for non bound fields
			if( ! $_ -> BoundTo() )
			{
				$Tmp = $_ -> Default();
				$Tmp =~ s/\{(.*?)\}/${$ThisRef}{$1}/ge;
				$_ -> SetValue( $Tmp );
			}

			# Href processing
			if( $_ -> HREF() )
			{
				$Tmp = $_ -> HREF();
				$Tmp =~ s/\{(.*?)\}/${$ThisRef}{$1}/ge;

				$_ -> SetValue( '<A HREF="' .
						$Tmp .
						'">' .
						$_ -> Value() .
						'</A>' );
			}
		}

		# Make a snapshot of this row
		$ThisRow -> { fields } = [];
		$ThisRow -> { names } = {};
		$ThisRow -> { boundto } = {};

		for( @{ $Self -> ListFields() } )
		{
			$Self -> _AddFieldToHREF( $ThisRow, $_ );
		}

		push( @{ $Self -> UseData() -> { rows } }, $ThisRow );
	}

	#print $MagicQ;

	$Self -> _DisplayForm();
}

sub _WhatToDo
{
	# Look at the opts and decide what to do
	my $Self = shift;
	my $ActVal = $Self -> Opts() -> { $Self -> _ACTION() };
	my $ButVal = $Self -> Opts() -> { $Self -> _Button() };

	#print $ActVal . "::" . $ButVal . "<BR>";
	if( $ButVal eq $Self -> DeleteButton() )
	{
		# If they pushed delete then ask if they are sure
		return( "ASKDELETE" );
	}elsif( $ActVal eq "DELETE" )
	{
		# They said yes?
		if( $ButVal eq $Self -> DeleteYesButton() )
		{
			return( "DELETE" );
		}else
		{
			# Nope
			return( "NODELETE" )
		}
	}elsif( $ActVal eq "UPDATE" )
	{
		return( "UPDATE" );
	}elsif( $ActVal eq "ADD" )
	{
		return( "ADD" );
	}elsif( $Self -> PKeyVals() ) ### FIX ME
	{
		return( "ASKUPDATE" );
	}else
	{
		# Nothing means askadd
		return( "ASKADD" );
	}
}

sub _WhatForm
{
	# Just a logic filter for the above
	my $Self = shift;
	my $Do = $Self -> _WhatToDo();

	# This will tell what form to display and the method it should do
	if( grep( $_ eq $Do, 'ASKADD',
				'ADD',
				'UPDATE',
				'DELETE',
				'NODELETE' ) )
	{
		return( "ADD" );
	}elsif( $Do eq 'ASKUPDATE' )
	{
		return( "UPDATE" );
	}elsif( $Do eq "ASKDELETE" )
	{
		# Munge the PKEYS out of the query
		$Self -> PKeyVals( [] );
		for( @{ $Self -> PKeys } )
		{
			push( @{ $Self -> PKeyVals() }, $Self -> Opts() -> {
					$Self -> _HTMLNAME() . '.pkeys.' . $_ }
				);
		}
		return( "DELETE" );
	}

	return( undef );
				
}

sub _FillFields
{
	# Call proper filler based on action to take
	my $Self = shift;
	my $Do = $Self -> _WhatForm();

	# If askadd or just updated or just deleted, then fill defaults
	if( $Do eq 'ADD' )
	{
		$Self -> _FillFieldsAdd();
	}elsif( $Do eq "UPDATE" )
	{
		$Self -> _FillFieldsUpdate();
	}elsif( $Do eq "DELETE" )
	{
		$Self -> _FillFieldsUpdate();
	}
}

sub _FillFieldsUpdate
{
	# This will get values from an SQL query
	my $Self = shift();

	my @PKeys = @{ $Self -> PKeys() };
	my @Fields = $Self -> _SELECTFIELDS();
	my @Values;

	# Generate the select query
	my $MagicQ = 'select ' . join( ", ", @Fields ) .
		' from ' . $Self -> Table() .
		' where ' .
		join( " AND ", map( $_ . "=?", @PKeys ) );

	my $sth = $Self -> DBH() -> prepare( $MagicQ );
	$sth -> execute( @{ $Self -> PKeyVals() } );

	my $Ref = $sth -> fetchrow_hashref();

	if( ! defined( $Ref ) )
	{
		$Self -> AddMessage( "Row now found" );
		# Should break it down a bunch here
		return( undef );
	}
	for( keys( %{ $Ref } ) )
	{
		$Self -> getnamedfield( $_ ) -> SetValue( $Ref -> { $_ } );
	}
}

sub _FillFieldsAdd
{
	my $Self = shift;

	for( @{ $Self -> ListFields() } )
	{
		$_ -> SetDefault();
	}
}

sub _FillFieldsBad
{
	# This will get values from the environment
}

sub _DisplayForm
{
	my $Self = shift;

	my $Ret;

	# Performs the actual form display
	if( ! $Self -> Template() )
	{
		$Self -> Template( Site::Template -> new() );
		$Self -> Template() -> Template( $Self -> _DefaultTemplate() );
	}

	$Self -> Template() -> registerdefault( $Self -> UseData() );
	$Self -> Template() -> display();
}

sub _DisplayDelete
{
	my $Self = shift;
	if( ! $Self -> DeleteTemplate() )
	{
		$Self -> DeleteTemplate( Site::Template -> new() );
		$Self -> DeleteTemplate() ->
			Template( $Self -> _DefaultDeleteTemplate() );
	}

	$Self -> DeleteTemplate() -> registerdefault( $Self );
	$Self -> DeleteTemplate() -> display();
}

sub _DefaultTemplate
{
	# Return a default template
	return( qq{
<TABLE BORDER=0>
<TR>
<t:loop name=headers>
<TD><t:var name=header/></tD>
</t:loop>
</TR>

<t:loop name=rows>
<TR>
<t:loop name=fields>
<TD VALIGN=TOP><t:var name=display/></TD>
</t:loop>
</TR>
</t:loop>

</TABLE>
} );

}

sub _DefaultDeleteTemplate
{
	return( qq{<t:var name=HTMLBegin/>
Delete it?

<TABLE BORDER=0>
<t:loop name=HTMLFields>
<TR>
<TD><t:iftrue name=HTMLLabel><B><t:var name=HTMLLabel/>:</B></t:iftrue></TD>
<TD><t:var name=HTMLDisplay/></TD>
</TR>
</t:loop>
</TABLE>

<t:var name=HTMLDeleteYes/>
<t:var name=HTMLDeleteNo/>
<t:var name=HTMLEnd/>
} );
}

sub HTMLBegin
{
	my $Self = shift;
	my $Action = $Self -> FormAction();
	my $Do = $Self -> _WhatForm();

	my $Ret = qq{<FORM ACTION=$Action METHOD=GET>
} . '<INPUT TYPE=HIDDEN NAME=' . $Self -> _ACTION() .
	' VALUE="' . $Do . '">';

	if( $Do ne "ADD" )
	{
		# Add pkey vals
		my @Vals = @{ $Self -> PKeyVals() };
		for( @{ $Self -> PKeys() } )
		{
			$Ret .= '<INPUT TYPE=HIDDEN NAME=' .
				$Self -> _HTMLNAME() .
				'.pkeys.' .
				$_ .
				' VALUE="' .
				Site::Page ->
					encode_entities( shift( @Vals ) ) .
				'">';
		}
	}

	# Spew forth the carrables!
	for ( keys( %{ $Self -> Carry() } ) )
	{
		$Ret .= '<INPUT TYPE=HIDDEN NAME=' .
			$_ .
			' VALUE="' .
			Site::Page ->
				encode_entities( $Self -> Carry() -> { $_ } ) .
			'">';
	}

	return( $Ret );
}


sub HTMLEnd
{
	return( qq{</FORM>} );
}

sub HTMLLabel
{
	my $Self = shift;
	my $Fname = shift;

	return( $Self -> getnamedfield( $Fname ) -> HTMLLabel() );
}

sub HTMLField
{
	my $Self = shift;
	my $Fname = shift;

	return( $Self -> getnamedfield( $Fname ) -> HTMLField() );
}

sub HTMLAdd
{
	# The add button, if applicable
	my $Self = shift;

	if($Self -> _WhatForm() eq "ADD" )
	{
		return( '<INPUT TYPE=SUBMIT NAME=' . 
			$Self -> _Button() .
			' VALUE="' . 
			$Self -> AddButton() .
			'">' );
	}
	return( undef );
}

sub HTMLEdit
{
	# The edit button
	my $Self = shift;
	if( $Self -> _WhatForm() eq "UPDATE" )
	{
		return( '<INPUT TYPE=SUBMIT NAME=' . 
			$Self -> _Button() .
			' VALUE="' . 
			$Self -> EditButton() .
			'">' );
	}

	return( undef );
}

sub HTMLDelete
{
	# The delete button
	# Only on the update form
	my $Self = shift;

	if( $Self -> _WhatForm() eq "UPDATE" )
	{
		return( '<INPUT TYPE=SUBMIT NAME=' . 
			$Self -> _Button . 
			' VALUE="' . $Self -> DeleteButton() . '">' ); 
	}
	return( undef );
}

sub HTMLDeleteYes
{
	my $Self = shift();

	return( '<INPUT TYPE=SUBMIT NAME=' . 
			$Self -> _Button . 
			' VALUE="' . $Self -> DeleteYesButton() . '">' ); 
}

sub HTMLDeleteNo
{
	my $Self = shift();

	return( '<INPUT TYPE=SUBMIT NAME=' . 
			$Self -> _Button . 
			' VALUE="' . $Self -> DeleteNoButton() . '">' ); 
}

sub HTMLDisplay
{
	my $Self = shift;
	return( $Self -> getnamedfield( shift() ) -> displayValue() );
}

sub _Button
{
	my $Self = shift;

	return( $Self -> _HTMLNAME() . ".__.button" );
}

sub AddMessage
{
	my $Self = shift;
	$Self -> { MESSAGE } .= shift() . "\n";
}

sub _FillPKeys
{
	my $Self = shift;
	my @Arr;
	my $IsSet = 0;

	for( @{ $Self -> _HTMLPKeys() } )
	{
		push( @Arr, $Self -> Opts() -> { $_ } );
		if( defined( $Arr[ -1 ] ) )
		{
			$IsSet = 1;
		}
	}

	if( $IsSet )
	{
		$Self -> { PKEYVALS } = \@Arr;
	}
}

sub _HTMLPKeys
{
	# This will be configurable later, for now the pkeys are not fully
	# qualified
	my $Self = shift;

	return( $Self -> PKeys() );
}

sub HTMLHeaders
{
	my $Self = shift;
	# Make an array for the header
	my @ret;

	for( @{ $Self -> ListFields() } )
	{
		#print $_ -> Name() . "\n";
		#print Dumper( $_ ) . "\n";

		push( @ret, {
				HTMLHeader =>
					$Self -> HTMLHeader( $_ -> Name() ),
			} );

	}

	return( \@ret );
}

sub HTMLHeader
{
	my $Self = shift;
	my( %NewOpts );
	my $Field = $Self -> getnamedfield( shift() );
	my $SortOrder;

	if( ! $Field )
	{
		return( undef );
	}

	if( ! $Field -> BoundTo() )
	{
		return( $Field -> HTMLLabel() );
	}

	# Fill out the stuff to carry
	$NewOpts{ 'query' } = Site::Page -> escape_uri( $Self -> Query() );

	$NewOpts{ 'order' } = Site::Page -> escape_uri( join( ",",
					$Field -> BoundTo(),
					grep( $_ ne $Field -> BoundTo(),
						@{ $Self -> Order() || [] } ) ) );

	$NewOpts{ 'where' } = Site::Page -> escape_uri( $Self -> Where() );

	if( $Self -> WhereArgs() )
	{
		$NewOpts{ 'whereargs' } = Site::Page -> escape_uri( join( ",", @{ $Self -> WhereArgs() } ) );
	}

	# really inefficient right now!!
	if( $Self -> Order() )
	{
		my $Cnt = 0;
		for( @{ $Self -> Order() } )
		{
			$Cnt++;
			s/\s.*$//;

			if( lc( $_ ) eq lc( $Field -> BoundTo() ) )
			{
				$SortOrder = "(" . $Cnt . ")";
			}
		}
	}

	return( $SortOrder . '<A HREF="' .
		$Self -> FormAction() .
		'?' .
		$Self -> _ACTION() .
		'=SORT&' .
		join( "&", map( $Self -> _HTMLNAME() . ".sort." . $_ .
					'=' . $NewOpts{ $_ }, keys( %NewOpts ) ) ) .
		'">' .
		$Field -> HTMLLabel() .
		'</A>' );
}

sub _LoadTableFromOpts
{
	# Override table options if passed in the URI
	my $Self = shift;
	my $Opts = $Self -> Opts();
	my $Sname = $Self -> _HTMLNAME() . ".sort.";

	if( $Opts -> { $Self -> _ACTION() } eq "SORT" )
	{
		$Self -> { QUERY } = $Opts -> { $Sname . "query" };
		$Self -> { WHERE } = $Opts -> { $Sname . "where" };

		$Self -> { ORDER } = [];
		for( split( /,/, $Opts -> { $Sname . "order" } ) )
		{
			push( @{ $Self -> { ORDER } }, $_ );
		}

		$Self -> { WHEREARGS } = [];
		for( split( /,/, $Opts -> { $Sname . "whereargs" } ) )
		{
			push( @{ $Self -> { WHEREARGS } }, $_ );
		}
	}
}

sub _AddFieldToHREF
{
	my $Self = shift;
	my $href = shift;
	my $Field = shift;

	my( $useref, $fieldref );
	
	$useref = {
			field => $Field -> HTMLField(),
			label => $Field -> HTMLLabel(),
			display => $Field -> HTMLDisplay(),
			header => $Self -> HTMLHeader( $Field -> Name() )
		};

	push( @{ $href -> { fields } }, $useref );
	$href -> { names } = $useref;
	$href -> { boundto } = $useref;
}

1;
